/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "BoundTypes.h"

#include "ZeroAccess.h"
#include "ClampAccess.h"
#include "MirrorAccess.h"
#include "BoundAccess.h"

/**
 * returns true if type is a valid BoundType
 */
bool isValidBoundType(BoundType type)
{

	switch(type)
	{
		case ZERO:
		case CLAMP:
		case MIRROR:
		case RENORMALIZE:
			return true;
	}
	
	return false;
}

/**
 * Returns the string equivalent of type
 */
const char * boundTypeToString(BoundType type)
{
	switch(type)
	{
		case ZERO: return "zero";
		case CLAMP: return "clamp";
		case MIRROR: return "mirror";
		case RENORMALIZE: return "renormalize";
	}

	return "UNKNOWN";
}

/**
 * Returns a new pointer to the requested type
 */
BoundAccess * getBoundType(BoundType type)
{
	switch(type)
	{
		case ZERO: return new ZeroAccess;
		case CLAMP: return new ClampAccess;
		case MIRROR: return new MirrorAccess;
		case RENORMALIZE: return new ZeroAccess;
	}

	return 0;
}
