/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#ifndef FILTER_H
#define FILTER_H

class ImageSlice;
class BoundAccess;

#include <cmath>

/**
 * This is the base filter class.  It provides an interface
 * that requires that a calculate function be implemented.
 *
 * The blur function will use the calculate function to blure
 * the ImageSlice to a given radius
 *
 * The resample function will resample a given ImageSlice
 * using the calculate function.
 */
class Filter
{
	public:
		Filter(double r) { radius = fabs(r); };

		virtual bool blur(ImageSlice & input, 
					ImageSlice & output, 
					unsigned int radius,
					BoundAccess & accessor,
					bool renormalize = false);

		virtual bool resample(ImageSlice & input, 
					ImageSlice & output, 
					unsigned int newWidth,
					BoundAccess & accessor,
					bool renormalize = false);

	protected:
	
		virtual double calcValue(double value) = 0;
		double radius;
};

#endif // FILTER_H
