/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "FilterTypes.h"
#include "SpikeFilter.h"
#include "BoxFilter.h"
#include "TentFilter.h"
#include "GaussianFilter.h"
#include "BSplineCubicFilter.h"

/**
 * returns true if type is a valid
 * FilterType
 */
bool isValidFilter(FilterType type)
{
	switch(type)
	{
		case SPIKE:
		case BOX:
		case TENT:
		case GAUSSIAN:
		case BSPLINECUBIC:
			return true;
	}
	return false;
}

/**
 * Returns the string equivalent of type
 */
const char * filterToString(FilterType type)
{
	switch(type)
	{
		case SPIKE: return "spike";
		case BOX: return "box";
		case TENT: return "tent";
		case GAUSSIAN: return "guassian";
		case BSPLINECUBIC: return "b-spline cubic";
	}

	return "UNKNOWN";
}

/**
 * returns a new Filter of type type
 */
Filter * getFilter(FilterType type)
{
	switch(type)
	{
		case SPIKE: return new SpikeFilter;
		case BOX: return new BoxFilter;
		case TENT: return new TentFilter;
		case GAUSSIAN: return new GaussianFilter;
		case BSPLINECUBIC: return new BSplineCubicFilter;
	}

	return 0;
}

