/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#ifndef IMAGE_H
#define IMAGE_H

#include <string>
#include <vector>
using namespace std;

class RGBValue;
class TargaImage;
class ImageSlice;

/**
 * This image class is a proxy for the TargaImage.
 * It loads from a targa image and converts all values
 * into floating point values so that calculations
 * can be carried out easily
 *
 * It also provides functions for getting and putting
 * ImageSlices
 */
class Image
{
	public:
		Image(unsigned int width = 0, unsigned int height = 0);
		~Image();

		bool load(const string & file);
		bool write(const string & file);
	
		bool load(const TargaImage & image);
		TargaImage * buildTarga();

		unsigned int getWidth() const { return width; };
		unsigned int getHeight() const { return height; };

		//const RGBValue & operator()(unsigned int x, unsigned int y) const;
		//RGBValue & operator()(unsigned int x, unsigned int y);

		const RGBValue & get(unsigned int x, unsigned int y) const;
		RGBValue & get(unsigned int x, unsigned int y);

		bool getRow(unsigned int row, ImageSlice & out);
		bool putRow(unsigned int row, const ImageSlice & in);
		bool getCol(unsigned int col, ImageSlice & out);
		bool putCol(unsigned int col, const ImageSlice & in);

	private:
		Image(const Image & toCopy) { };
		const Image & operator=(const Image & toCopy) { };

		unsigned int width;
		unsigned int height;

		RGBValue * values;
};

#endif // IMAGE_H
