/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "Image.h"
#include "BoundAccess.h"
#include "Filter.h"
#include "ImageSlice.h"

/**
 * Blurs the image first horizontally then vertically
 */
bool blurImage(Image & in, Image & out, 
		BoundAccess & access, Filter & filter, 
		unsigned int radius, bool renormalize = false)
{
	ImageSlice inSlice;
	ImageSlice outSlice;

	if(in.getWidth() != out.getWidth() || in.getHeight() != out.getHeight())
		return false;

	Image temp(in.getWidth(), in.getHeight());
	
	// columns first
	for(unsigned int i = 0; i < in.getWidth(); i++)
	{	
		in.getCol(i, inSlice);
		filter.blur(inSlice, outSlice, radius, access, renormalize);
		temp.putCol(i, outSlice);
	}

	// rows second
	for(unsigned int i = 0; i < in.getHeight(); i++)
	{
		temp.getRow(i, inSlice);
		filter.blur(inSlice, outSlice, radius, access, renormalize);
		out.putRow(i, outSlice);
	}

	return true;
}

/**
 * resamples the image
 */
bool resampleImage(Image & in, Image & out, 
		BoundAccess & access, Filter & filter, bool renormalize = false)
{
	ImageSlice inSlice;
	ImageSlice outSlice;

	Image temp(in.getWidth(), out.getHeight());
	
	// columns first
	for(unsigned int i = 0; i < in.getWidth(); i++)
	{	
		in.getCol(i, inSlice);
		filter.resample(inSlice, outSlice, out.getHeight(), access, renormalize);
		temp.putCol(i, outSlice);
	}

	// rows second
	for(unsigned int i = 0; i < temp.getHeight(); i++)
	{
		temp.getRow(i, inSlice);
		filter.resample(inSlice, outSlice, out.getWidth(), access, renormalize);
		out.putRow(i, outSlice);
	}

	return true;
}

