/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "SpikeFilter.h"

#include "ImageSlice.h"
#include "BoundAccess.h"

#include <cmath>

/**
 * Spike filters can't really blur. but the
 * function needs to be implemented
 */
bool SpikeFilter::blur(ImageSlice & input, 
	ImageSlice & output, 
	unsigned int radius,
	BoundAccess & accessor,
	bool renormalize)
{
	// basically a straight copy from input to output
	output.setLength(input.getLength());
	
	for(unsigned int i = 0; i < input.getLength(); i++)
		output[i] = input[i];

	return true;	
}

/**
 * The simplest resampling method.  This implements
 * the nearest neighbor algorithm
 */
bool SpikeFilter::resample(ImageSlice & input, 
	ImageSlice & output, 
	unsigned int newWidth,
	BoundAccess & accessor,
	bool renormalize)
{
	// nearest neighbor
	output.setLength(newWidth);

	double step = static_cast<double>(input.getLength()) / static_cast<double>(newWidth);
	
	double currentStep = 0;
	for(unsigned int i = 0; i < newWidth; i++, currentStep += step)
	{
	//	cout << "i: " << i << " : " << currentStep << endl;
		if(currentStep - floor(currentStep) < 0.5)
			output[i] = input[static_cast<unsigned int>(floor(currentStep))];
		else 
			output[i] = input[static_cast<unsigned int>(ceil(currentStep))];
	}
			
	return true;
}

/**
 * Again, required but doesn't do anything
 */
double SpikeFilter::calcValue(double value)
{
	return 0.0;
}
