/**
 * This file is part of cs559ImageProcessing
 * 
 * cs559ImageProcessing is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cs559ImageProcessing is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with cs559ImageProcessing.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2010 James Hill
 */

#include "Image.h"
#include "ZeroAccess.h"
#include "BSplineCubicFilter.h"
#include "ImageOperations.h"
#include "BoundTypes.h"
#include "RGBValue.h"
#include "libtarga.h"
#include <stdlib.h>
#include <cmath>
#include <iostream>
#include <string>

using namespace std;

void splat(Image & splat, Image & out, RGBValue & color, int x, int y, int r);
void genNegative(Image & input, Image & output);
void printUsage();

int main(int argc, char** argv)
{
	if(argc < 3)
	{
		cout << "ERROR: Invalid command line arguments" << endl;
		printUsage();
		return -1;
	}

	Image image;

	if(image.load(string(argv[1])) != true)
	{
		cout << "ERROR: Unable to load [" << argv[1] << "] : " << tga_error_string(tga_get_last_error()) << endl;
		printUsage();
		return -1;
	}
	
	Image hf(image.getWidth(), image.getHeight());
	Image out(image.getWidth(), image.getHeight());

	genNegative(image, hf);

	Image brush;
	if(!brush.load("image/BigBrush.tga"))
	{
		cout << "ERROR: Failed to load BigBrush.tga: " << tga_error_string(tga_get_last_error()) << endl;
		printUsage();
		return -1;
	}
	for(int i = 0; i < image.getHeight(); i+=3)
	{
		for(int j = 0; j < image.getWidth(); j+=2)
		{
			splat(brush, out, image.get(j,i), j, i, 6);
		}
	}

	// look through the high frequency negative to find transitions and then put splats around it
	Image little;
	if(!little.load("image/LittleBrush.tga"))
	{
		cout << "ERROR: Failed to load LittleBrush.tga: " << tga_error_string(tga_get_last_error()) << endl;
		printUsage();
		return -1;
	}

	int last = hf.get(0,0).r;
	for(int i = 0; i < hf.getHeight(); i+=3)
	{
		for(int j = 0; j < hf.getWidth(); j+=3)
		{
			if(fabs(hf.get(j,i).r - last) > 40)
				splat(little, out, image.get(j,i), j, i, 0);
			
			last = hf.get(j,i).r; // comment out for cool effect
		}
	}

	out.write(string(argv[2]));
}

void printUsage()
{
	cout << endl;
	cout << "\tpainting input output" << endl;
	cout << endl;
	cout << "\t\tinput  - File name of input image" << endl;
	cout << "\t\toutput - File name of output image" << endl;
	cout << endl;	
}

void splat(Image & splat, Image & out, RGBValue & color, int x, int y, int r)
{
	if(r != 0)
	{
		x += (rand() % r - r/2);
		y += (rand() % r - r/2);
	}
	for(int c = 0; c < splat.getWidth(); c++)
	{
		for(int r = 0; r < splat.getHeight(); r++)
		{
			int iX = c + x - splat.getWidth() / 2;
			int iY = r + y - splat.getHeight() / 2;
			if(splat.get(c,r).r == 0 && iX >=0 && iX < out.getWidth() &&
				iY >= 0 && iY < out.getHeight())
				out.get(iX,iY) = color;
		}
	}
}

void genNegative(Image & input, Image & output)
{
	BoundAccess * za = new ZeroAccess;
	Filter *f = new BSplineCubicFilter;
	Image temp(input.getWidth(), input.getHeight());
	blurImage(input, temp, *za, *f, 20, true);

	for(int i = 0; i < input.getWidth(); i++)
	{
		for(int j = 0; j < input.getHeight(); j++)
		{
			output.get(i,j).r = input.get(i,j).r - temp.get(i,j).r;
			output.get(i,j).g = input.get(i,j).g - temp.get(i,j).g;
			output.get(i,j).b = input.get(i,j).b - temp.get(i,j).b;

			double average = (output.get(i,j).r + output.get(i,j).g + output.get(i,j).b) / 3.0;
		
			output.get(i,j).r = average;
			output.get(i,j).g = average;
			output.get(i,j).b = average;
			
		}
	}
	
	delete f;
	delete za;
}
